﻿Type=Class
Version=4.5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'WebSocket class: weatherchart.bas
'Connect to the server as websocket. Press button Request to obtain Weather Information in gauges.
'Set the Weather Underground API Key and the Lat,Lon position.
Sub Class_Globals
	Private ws As WebSocket
	' Constants
	Private const CAPIKEY As String = "add your api key"	
	Private const CLATLON As String = "53.551086,9.993682"	
	' BUTTON JQUERYELEMENTS
	Private apikey As JQueryElement
	Private latlon As JQueryElement
	Private lastupdate As JQueryElement
	Private weatherinfo As JQueryElement
	Private btnWeatherRequest As JQueryElement
	'
	Private WeatherInformation  As String
End Sub

Public Sub Initialize
'	
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	Log($"New connection Start"$)
	DateTime.DateFormat = "dd.MM.yyyy"
	ws = WebSocket1
	Main.ConnectedBrowsers.Put(Me, "")
	' Set the default position and apikey in index.html input fields. Note that SetVal must be used.
	apikey.SetVal(CAPIKEY)
	latlon.SetVal(CLATLON)
	'Show the chart with empty data ($"0;0;0;0"$) or with test data ($"23;73;1014;3"$)
	Load_Chart($"0;0;0;0"$)
	lastupdate.SetText($"Press Request to obtain Weather Information"$)
	weatherinfo.SetHTML($"No Weather Information.<hr>Press Request to obtain Weather Information"$)
	ws.flush
	Log($"New connection with ID ${ws.Session.Id}"$)
End Sub

Private Sub WebSocket_Disconnected
	Main.ConnectedBrowsers.Remove(Me)
End Sub

' Place a request to weather underground
' For Tests:
' Load_Chart($"23;60;1022;8"$)
' weatherinfo.SetText($"Additional Weather Information"$)
' Return
Private Sub btnWeatherRequest_Click(params As Map)
	Dim fapikey As Future = apikey.GetVal
	Dim flatlon As Future = latlon.GetVal
	lastupdate.SetText($"${DateTime.Time(DateTime.Now)} Weather Underground request started for position ${flatlon.value}."$)
	ws.flush
	JobRequest($"http://api.wunderground.com/api/${fapikey.value}/conditions/q/${flatlon.value}.json"$)
	Log($"Weather Underground request started for position ${flatlon}"$)
End Sub

'Private Sub btnweatherinfo_Click(params As Map)
'
'End Sub

#Region HTTPJOB
' Submit a job request with the given url. The sub jobdone handles the result.
Private Sub JobRequest(url As String)
	' HttpJob init and download
	Dim j As HttpJob		' from the library jHttpUtils2
   	j.Initialize("j", Me)
	j.Download(url)
End Sub

' Handle the result of the weather underground request. Returned from weather underground is a json string with the weather information.
Private Sub JobDone(Job As HttpJob)
	' Parse the result
	If Job.Success Then
		'Log(Job.GetString)
		ParseWeatherData(Job.GetString)
		Log($"Weather Underground request sucessfully completed."$)
   	Else
		Log($"Weather request error getting data from Weather Underground ${Job.ErrorMessage}"$)
   	End If
   	Job.Release
End Sub
#End Region

#Region JSON
' Parse the weather data given in json format, assign selected values to either global vars or set the gauges values direct.
' For tests, the data can be read from a file
Private Sub ParseWeatherData(jsonstring As String)
	Dim notes As StringBuilder
	Dim parser As JSONParser
	Dim fn As String = $"${latlon.GetVal.Value}.json"$
	notes.Initialize
	WeatherInformation = "No Weather Information"
	parser.Initialize(jsonstring)
	Dim root As Map = parser.NextObject
	Dim current_observation As Map = root.Get("current_observation")
	Dim display_location As Map = current_observation.Get("display_location")
	Dim observation_location As Map = current_observation.Get("observation_location")
	'
	Dim observation_time_rfc822 As String = current_observation.Get("observation_time_rfc822")
'	lblWeatherStatus.Text = $"Weather observed at ${observation_time_rfc822}"$
	
	Dim temp_c As Double = current_observation.Get("temp_c")
'	GaugeTemperature.Value = temp_c

	Dim pressure_mb As String = current_observation.Get("pressure_mb")
'	GaugeAirpressure.Value = pressure_mb

	Dim wind_gust_kph As Double = current_observation.Get("wind_gust_kph")
	Dim wind_bft As Int = WindSpeedToBeaufort(wind_gust_kph)
	'GaugeWindSpeed.Value = wind_gust_kph
'	GaugeWindSpeed.Value = WindSpeedToBeaufort(wind_gust_kph)

	Dim wind_degrees As Int = current_observation.Get("wind_degrees")
'	GaugeWindDirection.Value = wind_degrees
		
	Dim relative_humidity As String = current_observation.Get("relative_humidity")
	relative_humidity = relative_humidity.replace("%","")
'	GaugeHumidity.Value = relative_humidity.replace("%","")
	
	Dim precip_today_metric As String = current_observation.Get("precip_today_metric")
'	GaugePrecip.Value = precip_today_metric

	' Set the chartdata as string delimited with ;. Do no put a delimiter at text end.	
	Dim chartdata As String = $"${temp_c};${relative_humidity};${pressure_mb};${wind_bft}"$
	Load_Chart(chartdata)

	' Add some notes
	Dim cr As String = "<br>"
	notes.append($"Location:${cr}${display_location.Get("full")}"$).append(cr).append(cr)
	notes.Append($"Observation Time & Location:${cr}${observation_time_rfc822}${cr}${observation_location.Get("full")}"$).Append(cr).Append(cr)
	notes.Append($"<hr>"$)
	notes.Append($"Weather:${cr}${current_observation.Get("weather")}"$).Append(cr)
	notes.Append($"Wind:${cr}${wind_degrees} deg"$).Append(cr)
	notes.Append($"Precip today:${cr}${precip_today_metric} mm"$).Append(cr)
	notes.Append($"${cr}"$)
	notes.Append($"Weather Underground data saved to ${fn}"$)
	notes.Append($"<hr>"$)
'	notes.Append($"Go https://www.wunderground.com to find more about Weather Undergroud."$)
	notes.Append($"Find more about <a href="https://www.wunderground.com">Weather Undergroud</a>."$)
	WeatherInformation = notes.ToString
	weatherinfo.SetHTML($"${WeatherInformation}"$)
	ws.flush
	' Save the retrieved json string to a file as pretty string
	Dim jsongen As JSONGenerator
	jsongen.Initialize(root)
	File.WriteString(File.DirApp, $"${fn}"$, jsongen.ToPrettyString(2))
	Log($"Weather Underground data parsed. The JSON file saved to ${fn}."$)
End Sub

' For tests
Private Sub TestSimulation 'ignore
	ParseWeatherData(File.ReadString(File.DirApp, "wunderground.dat"))
End Sub
#End Region

'The main sub to create and display the chart
Public Sub Load_Chart(data As String)
	Log($"${DateTime.time(DateTime.Now)}: Load Chart with ${data}"$)	

	'Define cols and rows. Temp, Hum, Pressure, Windspeed
	Dim colsT As List:colsT.Initialize
	Dim rowsT As List:rowsT.Initialize
	Dim colsH As List:colsH.Initialize
	Dim rowsH As List:rowsH.Initialize
	Dim colsP As List:colsP.Initialize
	Dim rowsP As List:rowsP.Initialize
	Dim colsW As List:colsW.Initialize
	Dim rowsW As List:rowsW.Initialize
	'Create the chartdata with the two sets of data
	CreateChartData(colsT, rowsT, colsH, rowsH, colsP, rowsP, colsW, rowsW, data)
	'Set the options for the two gauges
	Dim optionsT As Map:optionsT.Initialize
	Dim optionsH As Map:optionsH.Initialize
	Dim optionsP As Map:optionsP.Initialize
	Dim optionsW As Map:optionsW.Initialize
	CreateChartOptions(optionsT, optionsH, optionsP, optionsW)
	'Write the response back to the requestor
	Dim jg As JSONGenerator
	jg.Initialize(CreateMap("datat": CreateMap ("cols": colsT, "rows": rowsT), _ 
							"datah": CreateMap ("cols": colsH, "rows": rowsH), _  
							"datap": CreateMap ("cols": colsP, "rows": rowsP), _  
							"dataw": CreateMap ("cols": colsW, "rows": rowsW), _  
							"optionst": optionsT, _ 
							"optionsh":optionsH, _  
							"optionsp":optionsP, _  
							"optionsw":optionsW, _  
							"lastupdate":DateTime.Time(DateTime.Now)))
	'Log(jg.ToPrettyString(2))
	Dim jsonstring As String = $"${jg.ToString}"$
	'Draw the chart by calling the function drawCharts defined in the index.html script section
	ws.RunFunction("drawCharts", Array As String(jsonstring))
	'Update label showing the time of the last update and the interval
'	Dim refreshinterval As Int = 5	'Params.Get("measureinterval")
'	lastupdate.SetText($"Last update: ${DateTime.Time(DateTime.Now)}, Chart Refresh Interval: ${refreshinterval}s"$)
	lastupdate.SetText($"Last update: ${DateTime.Time(DateTime.Now)}"$)
	'Do notforget to flush which will show the updated values
	ws.Flush
End Sub

Private Sub CreateColumn(id As String, Label As String, dataType As String) As Map
	Return CreateMap("id": id, "label": Label, "type": dataType)
End Sub

Private Sub CreateRow(Values As List) As Map
	Dim vr As List
	vr.Initialize
	For Each o As Object In Values
		vr.Add(CreateMap("v": "" & o & ""))
		'vr.Add(CreateMap("v": o))
	Next
	Return CreateMap("c": vr)
End Sub

' Create the chartdata for temperature, humidity, airpressure, windbft
Private Sub CreateChartData(colsT As List, rowsT As List, _ 
							colsH As List, rowsH As List, _  
							colsP As List, rowsP As List, _  
							colsW As List, rowsW As List, _  
							data As String)
	'Log("CreateLineChartData :: Start")
	colsT.Add(CreateColumn("", "°C", "number"))
	colsH.Add(CreateColumn("", "%RH", "number"))
	colsP.Add(CreateColumn("", "mBar", "number"))
	colsW.Add(CreateColumn("", "Bft", "number"))
	Dim s() As String = Regex.Split(";", data)
	rowsT.Add(CreateRow(Array As Object(s(0))))
	rowsH.Add(CreateRow(Array As Object(s(1))))
	rowsP.Add(CreateRow(Array As Object(s(2))))
	rowsW.Add(CreateRow(Array As Object(s(3))))
	'Log("CreateLineChartData :: End")
End Sub

Private Sub CreateChartOptions (optionsT As Map, optionsH As Map, optionsP As Map, optionsW As Map)
	'Temp
	optionsT.Put("min", -10)
	optionsT.Put("max", 40)
	optionsT.Put("yellowFrom",30)
	optionsT.Put("yellowTo",35)
	optionsT.Put("redFrom", 35)
	optionsT.Put("redTo",40)
	optionsT.Put("minorTicks",5)
	optionsT.Put("greenColor","#CCFFCC")
    optionsT.Put("yellowColor","#FFFFCC")
    optionsT.Put("redColor","#F78181")
	'Hum
	optionsH.Put("min", 10)
	optionsH.Put("max", 100)
	optionsH.Put("greenFrom", 30)
	optionsH.Put("greenTo",60)
	optionsH.Put("minorTicks",10)
	optionsH.Put("greenColor","#CCFFCC")
    optionsH.Put("yellowColor","#FFFFCC")
    optionsH.Put("redColor","#F78181")
	'Pressure
	optionsP.Put("min", 950)
	optionsP.Put("max", 1050)
	optionsP.Put("greenFrom", 950)
	optionsP.Put("greenTo",1000)
	optionsP.Put("minorTicks",10)
	optionsP.Put("greenColor","#CCFFCC")
    optionsP.Put("yellowColor","#FFFFCC")
    optionsP.Put("redColor","#F78181")
	'Wind
	optionsW.Put("min", 0)
	optionsW.Put("max", 12)
	optionsW.Put("greenFrom", 0)
	optionsW.Put("greenTo",4)
	optionsW.Put("yellowFrom", 5)
	optionsW.Put("yellowTo",8)
	optionsW.Put("redFrom", 9)
	optionsW.Put("redTo",12)
	optionsW.Put("minorTicks",1)
	optionsW.Put("greenColor","#CCFFCC")
    optionsW.Put("yellowColor","#FFFFCC")
    optionsW.Put("redColor","#F78181")
End Sub

#Region WINDSPEED
' Convert WindSpeed to Beaufort
' Returns the Beaufort or -1 if windspeed not in range weather underground
Private Sub WindSpeedToBeaufort(wind As Double) As Int
	Dim Result As Int = -1
	'Windnames not used for now
	'Dim WindNames() As String
	'WindNames = Array As String("calm", "light air", "light breeze", "gentle breeze","moderate breeze","fresh breeze","strong breeze","near gale","gale","strong gale","storm","violent storm")
	' Array with max windspeed in kmh
	Dim windspeed() As Int
	windspeed = Array As Int(1,6,11,19,30,39,50,61,74,87,102,117,177,249,332,418,512)
 	' Get the windspeed from the array
 	For i = 1 To windspeed.Length - 1
		If (wind > windspeed(i - 1)) And (wind <= windspeed(i)) Then Result = i
	Next
 	Return Result
End Sub
#End Region
